# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import GroupMembershipV1


class IGroupMembershipsService(ABC):

    @abstractmethod
    def get_memberships(self, context: Optional[IContext], filter_params: FilterParams,
                        paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_membership_by_id(self, context: Optional[IContext], membership_id: str) -> Optional[GroupMembershipV1]:
        raise NotImplementedError()

    @abstractmethod
    def create_membership(self, context: Optional[IContext], membership: GroupMembershipV1) -> GroupMembershipV1:
        raise NotImplementedError()

    @abstractmethod
    def update_membership(self, context: Optional[IContext], membership: GroupMembershipV1) -> GroupMembershipV1:
        raise NotImplementedError()

    @abstractmethod
    def delete_membership_by_id(self, context: Optional[IContext], membership_id: str) -> Optional[GroupMembershipV1]:
        raise NotImplementedError()

    @abstractmethod
    def delete_memberships_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator

class GroupV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        title: Optional[str] = None,
        active_since: Optional[datetime] = None,
        owner_id: Optional[str] = None,
        group_active: Optional[bool] = True,

    ):
        self.id: str = id or IdGenerator.next_long()
        self.title = title
        self.active_since = active_since or datetime.now()
        self.owner_id = owner_id
        self.group_active = group_active or True

    def to_dict(self):
        return {
            "id": self.id,
            "title": self.title,
            "active_since": self.active_since.isoformat() if isinstance(self.active_since, datetime) else self.active_since,
            "owner_id": self.owner_id,
            "group_active": self.group_active,
        }


# -*- coding: utf-8 -*-
import datetime
from typing import List, Any


class SessionUserV1:
    def __init__(self, id: str = None, login: str = None, name: str = None, create_time: str = None,
                 time_zone: str = None, language: str = None,
                 theme: str = None, roles: List[str] = None, change_pwd_time: datetime.datetime = None,
                 sites: List[dict] = None, settings: Any = None, custom_hdr: Any = None,
                 custom_dat: Any = None):

        # Identification
        self.id = id
        self.login = login
        self.name = name
        self.create_time = create_time

        # User information
        self.time_zone = time_zone
        self.language = language
        self.theme = theme

        # Security info
        self.roles = roles
        self.change_pwd_time = change_pwd_time
        self.sites = sites
        self.settings = settings

        # Custom fields
        self.custom_hdr = custom_hdr
        self.custom_dat = custom_dat

    def to_dict(self):
        return {
            "id": self.id,
            "login": self.login,
            "name": self.name,
            "create_time": self.create_time,
            "time_zone": self.time_zone,
            "language": self.language,
            "theme": self.theme,
            "roles": self.roles,
            "change_pwd_time": self.change_pwd_time.isoformat() if self.change_pwd_time else None,
            "sites": self.sites,
            "settings": self.settings,
            "custom_hdr": self.custom_hdr,
            "custom_dat": self.custom_dat
        }



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_data.keys import IdGenerator

from eic_aichat_users.groups.data.GroupV1 import GroupV1
from eic_aichat_users.groups.logic.IGroupsService import IGroupsService
from eic_aichat_users.groups.persistence.IGroupsPersistence import IGroupsPersistence



class GroupsService(IGroupsService, IConfigurable, IReferenceable):
    _persistence: IGroupsPersistence = None
    def configure(self, config: ConfigParams):
        pass  

    def set_references(self, references: IReferences):
        self._persistence = references.get_one_required(
            Descriptor('aichatusers-groups', 'persistence', '*', '*', '1.0')
        )

    def get_groups(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging)

    def get_group_by_id(self, context: Optional[IContext], group_id: str) -> Optional[GroupV1]:
        return self._persistence.get_one_by_id(context, group_id)

    def create_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        group.id = group.id or IdGenerator.next_long()
        group.active_since = group.active_since or datetime.utcnow()
        return self._persistence.create(context, group)

    def set_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        group.active_since = group.active_since or datetime.utcnow()
        return self._persistence.set(context, group)

    def update_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        return self._persistence.update(context, group)

    def delete_group_by_id(self, context: Optional[IContext], group_id: str) -> Optional[GroupV1]:
        return self._persistence.delete_by_id(context, group_id)

    def delete_groups_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        self._persistence.delete_by_filter(context, filter_params)



# -*- coding: utf-8 -*-
from typing import List

import bottle
from pip_services4_commons.convert import JsonConverter
from pip_services4_commons.errors import UnauthorizedException
from pip_services4_http.auth.BasicAuthorizer import BasicAuthorizer
from pip_services4_http.auth.OwnerAuthorizer import OwnerAuthorizer
from pip_services4_http.auth.RoleAuthorizer import RoleAuthorizer


class AuthorizerV1:

    def __init__(self):
        self.__basic_auth = BasicAuthorizer()
        self.__role_auth = RoleAuthorizer()
        self.__owner_auth = OwnerAuthorizer()

    def anybody(self):
        return self.__basic_auth.anybody()

    def signed(self):
        return self.__basic_auth.signed()

    def owner(self, id_param: str = 'user_id'):
        return self.__owner_auth.owner(id_param)

    def owner_or_admin(self, id_param: str = 'user_id'):
        return self.__owner_auth.owner_or_admin(id_param)

    def site_roles(self, roles: List[str], id_param: str = 'site_id'):
        def inner():
            user = getattr(bottle.request, 'user', None)

            if user is None:
                raise UnauthorizedException(
                    None, 'NOT_SIGNED',
                    'User must be signed in to perform this operation'
                ).with_status(401)

            else:
                user.roles = getattr(user, 'roles', False) or []
                site_id = bottle.request.params['kwargs'].get(id_param)
                authorized = 'admin' in user.roles
                if site_id is not None and not authorized:
                    for role in roles:
                        authorized = authorized or (site_id + ':' + role) in user.roles

                if not authorized:
                    raise UnauthorizedException(
                        None, 'NOT_IN_SITE_ROLE',
                        'User must be site:' + ' or site:'.join(roles) + ' to perform this operation'
                    ).with_details('roles', roles).with_status(403)

        return inner

    def admin(self):
        return self.__role_auth.user_in_role('admin')

    def site_admin(self, id_param: str = 'site_id'):
        return self.site_roles(['admin'], id_param)

    def site_manager(self, id_param: str = 'site_id'):
        return self.site_roles(['admin', 'manager'], id_param)

    def site_user(self, id_param: str = 'site_id'):
        return self.site_roles(['admin', 'manager', 'user'], id_param)

    def site_admin_or_owner(self, user_id_param: str = 'user_id', site_id_param: str = 'site_id'):
        def inner():
            user = bottle.request.user
            if user is None:
                raise UnauthorizedException(
                    None, 'NOT_SIGNED',
                    'User must be signed in to perform this operation'
                ).with_status(401)

            else:
                user_id = dict(bottle.request.query.decode()).get(user_id_param) or JsonConverter.to_json(
                    bottle.request.json)
                if user_id is not None and user_id == user.user_id:
                    return
                else:
                    site_id = bottle.request.params.get(site_id_param)
                    authorized = 'admin' in user.roles or site_id + ':admin' in user.roles
                    if not authorized:
                        raise UnauthorizedException(
                            None, 'NOT_IN_SITE_ROLE',
                            'User must be site:admin to perform this operation'
                        ).with_details('roles', ['admin']).with_status(403)

        return inner




# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator


class AccountV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        login: Optional[str] = None,
        name: Optional[str] = None,
        language: Optional[str] = None,
        theme: Optional[str] = None,
        time_zone: Optional[str] = None,
        create_time: Optional[datetime] = None,
        deleted: Optional[bool] = False,
        active: Optional[bool] = True,
        about: Optional[str] = None,
        custom_hdr: Optional[Any] = None,
        custom_dat: Optional[Any] = None
    ):
        self.id: str = id or IdGenerator.next_long()
        self.login = login
        self.name = name
        self.create_time = create_time or datetime.now()
        self.deleted = deleted
        self.active = active
        self.about = about
        self.time_zone = time_zone
        self.language = language
        self.theme = theme
        self.custom_hdr = custom_hdr
        self.custom_dat = custom_dat

    def to_dict(self):
        return {
            "id": self.id,
            "login": self.login,
            "name": self.name,
            "language": self.language,
            "theme": self.theme,
            "time_zone": self.time_zone,
            "create_time": self.create_time.isoformat() if isinstance(self.create_time, datetime) else self.create_time,
            "deleted": self.deleted,
            "active": self.active,
            "about": self.about,
            "custom_hdr": self.custom_hdr,
            "custom_dat": self.custom_dat
        }



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from eic_aichat_users.groups.data.GroupV1 import GroupV1


class IGroupsService(ABC):

    @abstractmethod
    def get_groups(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_group_by_id(self, context: Optional[IContext], group_id: str) -> Optional[GroupV1]:
        raise NotImplementedError()

    @abstractmethod
    def create_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        raise NotImplementedError()

    @abstractmethod
    def set_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        raise NotImplementedError()

    @abstractmethod
    def update_group(self, context: Optional[IContext], group: GroupV1) -> GroupV1:
        raise NotImplementedError()

    @abstractmethod
    def delete_group_by_id(self, context: Optional[IContext], group_id: str) -> Optional[GroupV1]:
        raise NotImplementedError()

    @abstractmethod
    def delete_groups_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        raise NotImplementedError()



# -*- coding: utf-8 -*-

__all__ = ['UserPasswordV1', 'UserPasswordInfoV1', 'PasswordActivityTypeV1']

from .UserPasswordV1 import UserPasswordV1
from .UserPasswordInfoV1 import UserPasswordInfoV1
from .PasswordActivityTypeV1 import PasswordActivityTypeV1



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Any, Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import SessionV1


class ISessionsService(ABC):

    @abstractmethod
    def get_sessions(self, context: Optional[IContext], filter_params: FilterParams,
                     paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_session_by_id(self, context: Optional[IContext], session_id: str) -> Optional[SessionV1]:
        raise NotImplementedError()

    @abstractmethod
    def open_session(self, context: Optional[IContext], user_id: str, user_name: str,
                     address: str, client: str, user: Any, data: Any) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def store_session_data(self, context: Optional[IContext], session_id: str, data: Any) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def update_session_user(self, context: Optional[IContext], session_id: str, user: Any) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def close_session(self, context: Optional[IContext], session_id: str) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def close_expired_sessions(self, context: Optional[IContext]):
        raise NotImplementedError()

    @abstractmethod
    def delete_session_by_id(self, context: Optional[IContext], session_id: str) -> SessionV1:
        raise NotImplementedError()



# -*- coding: utf-8 -*-
import bottle

from typing import List
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_http.controller import RestOperations, RestController, HttpRequestDetector
from pip_services4_components.context import Context
from pip_services4_commons.convert import TypeCode, JsonConverter
from pip_services4_components.config import ConfigParams
from pip_services4_commons.errors import UnauthorizedException, BadRequestException

from eic_aichat_users.groups.data.GroupV1 import GroupV1
from eic_aichat_users.groups.logic.IGroupsService import IGroupsService
from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1
from eic_aichat_users.sessions.data import SessionV1
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService
from eic_aichat_users.accounts.data.AccountV1 import AccountV1
from eic_aichat_users.passwords.logic.IPasswordsService import IPasswordsService
from eic_aichat_users.sessions.logic.ISessionsService import ISessionsService
from eic_aichat_users.passwords.data.version1.UserPasswordInfoV1 import UserPasswordInfoV1
from eic_aichat_users.roles.logic.IRolesService import IRolesService
from eic_aichat_users.settings.logic.ISettingsService import ISettingsService

from google.oauth2 import id_token
from google.auth.transport import requests

from .SessionUserV1 import SessionUserV1


class SessionsOperations(RestOperations):
    __default_config = ConfigParams.from_tuples(
        'options.cookie_enabled', True,
        'options.cookie', 'x-session-id',
        'options.max_cookie_age', 365 * 24 * 60 * 60 * 1000
    )
        
    def __init__(self):
        super().__init__()
        self.__cookie: str = 'x-session-id'
        self.__cookie_enabled: bool = True
        self.__max_cookie_age: float = 365 * 24 * 60 * 60 * 1000
        self.__google_client_id: str = ""
        self._accounts_service: IAccountsService = None
        self._passwords_service: IPasswordsService = None
        self._sessions_service: ISessionsService = None
        self._roles_service: IRolesService = None
        self._settings_service: ISettingsService = None
        self._group_service: IGroupsService = None

        self._dependency_resolver.put("accounts-service", Descriptor('aichatusers-accounts', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("passwords-service", Descriptor('aichatusers-passwords', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("sessions-service", Descriptor("aichatusers-sessions", "service", "*", "*", "1.0"))
        self._dependency_resolver.put("roles-service", Descriptor('aichatusers-roles', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("settings-service", Descriptor('aichatusers-settings', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put('groups', Descriptor('aichatusers-groups', 'service', '*', '*', '1.0'))

    def configure(self, config):
        config = config.set_defaults(self.__default_config)
        super().configure(config)
        self._dependency_resolver.configure(config)

        self.__cookie_enabled = config.get_as_boolean_with_default('options.cookie_enabled', self.__cookie_enabled)
        self.__cookie = config.get_as_string_with_default('options.cookie', self.__cookie)
        self.__max_cookie_age = config.get_as_long_with_default('options.max_cookie_age', self.__max_cookie_age)
        self.__google_client_id = config.get_as_string_with_default('options.google_client_id', self.__google_client_id)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._accounts_service = self._dependency_resolver.get_one_required('accounts-service')
        self._passwords_service = self._dependency_resolver.get_one_required('passwords-service') 
        self._sessions_service = self._dependency_resolver.get_one_required("sessions-service")
        self._roles_service = self._dependency_resolver.get_one_required("roles-service")
        self._settings_service = self._dependency_resolver.get_one_required('settings-service')
        self._group_service = self._dependency_resolver.get_one_required('groups')


    def load_session(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------load_session() invoked")
        
        # parse headers first, and if nothing in headers get cookie
        session_id = bottle.request.headers.get('x-session-id')
            
        if session_id:
            try:
                session = self._sessions_service.get_session_by_id('facade', session_id)
                if session is None:
                    raise UnauthorizedException(
                        'facade',
                        'SESSION_NOT_FOUND',
                        'Session invalid or already expired.'
                    ).with_details('session_id', session_id).with_status(440)
                
                bottle.request.user_id = session.user_id
                bottle.request.user_name = session.user_name
                bottle.request.user = session.user
                bottle.request.session_id = session.id
                
            except Exception as err:
                return self._send_error(err)
            
    def get_sessions(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_sessions() invoked")

        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._sessions_service.get_sessions(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_session_by_id(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_session_by_id() invoked")

        try:
            res = self._sessions_service.get_session_by_id(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)


    def open_session(self, account: AccountV1, roles: List[str], ):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------open_session() invoked")

        try:
            session: SessionV1 = None
            # sites: List[SiteV1] = None
            passwordInfo: UserPasswordInfoV1 = None
            settings: ConfigParams = None

            # Retrieve sites for user

            site_roles = [] if not roles else list(filter(lambda x: x.find(':') > 0, roles))
            # site_ids = [] if not site_roles else list(map(lambda x: x[0:x.find(':')] if x.find(':') >= 0 else x, site_roles))

            # if len(site_ids) > 0:
            #     filter_params = FilterParams.from_tuples('ids', site_ids)
            #     page = self.__sites_client.get_sites(None, filter_params, None)
            #     sites = [] if page is None else page.data
            # else:
            #     sites = []

            password_info = self._passwords_service.get_password_info(context, account.id)

            settings = self._settings_service.get_section_by_id(context, account.id)

            # Open a new user session
            user = SessionUserV1(
                id=account.id,
                name=account.name,
                login=account.login,
                create_time=account.create_time,
                time_zone=account.time_zone,
                language=account.language,
                theme=account.theme,
                roles=roles,
                sites=[],
                # sites=list(map(lambda x: {'id': x.id, 'name': x.name}, sites)),
                settings=settings,
                change_pwd_time=None if password_info is None else password_info.change_time,
                custom_hdr=account.custom_hdr,
                custom_dat=account.custom_dat
            )

            address = HttpRequestDetector.detect_address(bottle.request)
            client = HttpRequestDetector.detect_browser(bottle.request)
            platform = HttpRequestDetector.detect_platform(bottle.request)

            session = self._sessions_service.open_session(context, account.id, account.name, address, client, user.to_dict(), None)
            return JsonConverter.to_json(session)
        except Exception as err:
            return self._send_error(err)

    def store_session_data(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------store_session_data() invoked")

        data = bottle.request.json or {}
        try:
            res = self._sessions_service.store_session_data(context, session_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def update_session_user(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------update_session_user() invoked")

        data = bottle.request.json or {}
        try:
            res = self._sessions_service.update_session_user(context, session_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def close_session(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------close_session() invoked")

        try:
            res = self._sessions_service.close_session(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def delete_session_by_id(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------delete_session_by_id() invoked")

        try:
            res = self._sessions_service.delete_session_by_id(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def signup(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------signup() invoked")

        try:
            signup_data = bottle.request.json if isinstance(bottle.request.json, dict) else JsonConverter.from_json(
                TypeCode.Map, bottle.request.json)
            account: AccountV1 = None
            invited: bool = False
            roles: List[str] = []

            # Validate password first
            # Todo: complete implementation after validate password is added

            # Create account
            new_account = AccountV1(
                None,
                login=signup_data.get('login') or signup_data.get('email'),
                name=signup_data.get('name'),
                language=signup_data.get('language'),
                theme=signup_data.get('theme'),
                time_zone=signup_data.get('time_zone')
            )

            account = self._accounts_service.create_account(context, new_account)

            # Create password for the account
            password = signup_data.get('password')

            self._passwords_service.set_password(context, account.id, password)

            # Activate all pending invitations
            email = signup_data.get('email')

            # invitations = self.__invitations_client.activate_invitations(context, email, account.id)
            # if invitations:
            #     # Calculate user roles from activated invitations
            #     for invitation in invitations:
            #         # Was user invited with the same email?
            #         invited = invited or email == invitation.invitee_email

            #         if invitation.site_id:
            #             invitation.role = invitation.role or 'user'
            #             role = invitation.site_id + ':' + invitation.role
            #             roles.append(role)

            # # Create email settings for the account
            # new_email_settings = EmailSettingsV1(
            #     id=account.id,
            #     name=account.name,
            #     email=email,
            #     language=account.language
            # )

            # if self.__email_settings_client is not None:
            #     if invited:
            #         self.__email_settings_client.set_verified_settings(context, new_email_settings)
            #     else:
            #         self.__email_settings_client.set_settings(context, new_email_settings)

            # Create user group
            group = GroupV1(
                title=f'User group for {account.name}',
                owner_id=account.id
            )

            self._group_service.create_group(context, group)

            return self.open_session(account, roles)
        except Exception as err:
            return self._send_error(err)

    def signup_validate(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------signup_validate() invoked")

        login = bottle.request.query.login

        if login:
            try:
                account = self._accounts_service.get_account_by_id_or_login(context, login)
                if account:
                    raise BadRequestException(
                        None, 'LOGIN_ALREADY_USED',
                        'Login ' + login + ' already being used'
                    ).with_details('login', login)

                return self._send_result("ok")

            except Exception as err:
                return self._send_error(err)
        else:
            return self._send_bad_request('Login is not specified')

    def signin(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------signin() invoked")

        json_data = bottle.request.json if isinstance(bottle.request.json, dict) else JsonConverter.from_json(
            TypeCode.Map, bottle.request.json)

        login = json_data.get('login')
        password = json_data.get('password')

        account: AccountV1 = None
        roles: List[str] = []

        # Find user account
        try:
            account = self._accounts_service.get_account_by_id_or_login(context, login)

            if account is None:
                raise BadRequestException(
                    None,
                    'WRONG_LOGIN',
                    'Account ' + login + ' was not found'
                ).with_details('login', login)
            
            login_provider = None
            if account.custom_dat and isinstance(account.custom_dat, dict):
                login_provider = account.custom_dat.get('login_provider')

            if login_provider == 'google':
                raise BadRequestException(
                    None,
                    'OAUTH_LOGIN_REQUIRED',
                    f'Account {login} is linked to Google. Please use Google login.'
                ).with_details('login_provider', 'google')

            # Authenticate user
            result = self._passwords_service.authenticate(context, account.id, password)
            # wrong password error is UNKNOWN when use http client
            if result is False:
                raise BadRequestException(
                    None,
                    'WRONG_PASSWORD',
                    'Wrong password for account ' + login
                ).with_details('login', login)

            # Retrieve user roles
            if self._roles_service:
                roles = self._roles_service.get_roles_by_id(context, account.id)
            else:
                roles = []

            return self.open_session(account, roles)
        except Exception as err:
            return self._send_error(err)
    
    def close_expired_sessions(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------close_expired_sessions() invoked")

        try:
            self._sessions_service.close_expired_sessions(context)
            return self._send_empty_result()
        except Exception as err:
            return self._send_error(err)
        
    def restore_session(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------restore_session() invoked")
        try:
            session_id = bottle.request.headers.get('x-session-id')

            session = self._sessions_service.get_session_by_id(context, session_id)

            # If session closed then return null
            if session and not session.active:
                session = None

            if session:
                return JsonConverter.to_json(session)
            else:
                return bottle.HTTPResponse(status=204)
        except Exception as err:
            return self._send_error(err)
        
    def signout(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------signout() invoked")
        if hasattr(bottle.request, 'session_id'):
            try:
                self._sessions_service.close_session(context, bottle.request.session_id)
                return bottle.HTTPResponse(status=204)
            except Exception as err:
                self._send_error(err)

        return bottle.HTTPResponse(status=204)

    def get_user_sessions(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_user_sessions() invoked")

        filter_params = self._get_filter_params()
        paging = self._get_paging_params()

        filter_params.set_as_object('user_id', user_id)

        sessions = self._sessions_service.get_sessions(context, filter_params, paging)
        return self._send_result(sessions.to_json())
        
    def get_current_session(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_current_session() invoked")

        # parse headers first, and if nothing in headers get cookie
        session_id = bottle.request.headers.get('x-session-id')

        session = self._sessions_service.get_session_by_id(context, session_id)
        return self._send_result(session)
    
    def login_with_google(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------login_with_google() invoked")

        try:
            bearer_token = bottle.request.headers.get('Authorization')
            token = bearer_token.split('Bearer ')[1]

            idinfo = id_token.verify_oauth2_token(
                token,
                requests.Request(),
                self.__google_client_id
            )

            email = idinfo["email"]
            name = idinfo.get("name")
            image = idinfo.get("picture")
            locale = idinfo.get("locale")
            roles: List[str] = []

            account = self._accounts_service.get_account_by_id_or_login(context, email)
            if account is None or account.id == "":
                # Create account
                new_account = AccountV1(
                    None,
                    login=email,
                    name=name,
                    language=locale,
                    custom_dat={
                        "login_provider": "google"
                    }
                )

                account = self._accounts_service.create_account(context, new_account)

            login_provider = None
            if account.custom_dat and isinstance(account.custom_dat, dict):
                login_provider = account.custom_dat.get('login_provider')

            if login_provider != 'google':
                raise BadRequestException(
                    None,
                    'USER_EXISTS',
                    f'Account {email} is already exists. Please use another login.'
                ).with_details('login_provider', 'google')

            return self.open_session(account, roles)

        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController, auth: AuthorizerV1):
        controller.register_route_with_auth('post', '/users/signup', None, auth.anybody(), lambda: self.signup())

        controller.register_route_with_auth('post', '/users/login/google', None, auth.anybody(), lambda: self.login_with_google())
        
        controller.register_route_with_auth('get', '/users/signup/validate', None, auth.anybody(), lambda: self.signup_validate())
        
        controller.register_route_with_auth('post', '/users/signin', None, auth.anybody(), lambda: self.signin())
        
        controller.register_route_with_auth('post', '/users/signout', None, auth.anybody(),lambda: self.signout())
        
        controller.register_route_with_auth('get', '/users/sessions', None, auth.admin(), lambda: self.get_sessions())
        
        controller.register_route_with_auth('post', '/users/sessions/restore', None, auth.signed(), lambda: self.restore_session())
        
        controller.register_route_with_auth('get', '/users/sessions/current', None, auth.signed(), lambda: self.get_current_session())
        
        controller.register_route_with_auth('get', '/users/:user_id/sessions', None, auth.signed(), lambda user_id: self.get_user_sessions(user_id))
        
        controller.register_route_with_auth('delete', '/users/:user_id/sessions/:session_id', None, auth.signed(), 
                                  lambda user_id, session_id: self.close_session(session_id))




# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator


class IStringIdentifiable:
    id: str


class GroupMembershipV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        profile_id: Optional[str] = None,
        group_id: Optional[str] = None,
        created: Optional[datetime] = None,
        active: Optional[bool] = True,
        member_since: Optional[datetime] = None
    ):
        self.id: str = id or IdGenerator.next_long()
        self.profile_id = profile_id
        self.group_id = group_id
        self.created = created or datetime.utcnow()
        self.active = active if active is not None else True
        self.member_since = member_since or datetime.utcnow()

    def to_dict(self):
        return {
            "id": self.id,
            "profile_id": self.profile_id,
            "group_id": self.group_id,
            "created": self.created.isoformat() if isinstance(self.created, datetime) else self.created,
            "active": self.active,
            "member_since": self.member_since.isoformat() if isinstance(self.member_since, datetime) else self.member_since
        }



# -*- coding: utf-8 -*-

__all__ = [
    'IAccountsPersistence', 'AccountsMemoryPersistence', 'AccountsMongoDbPersistence'
]

from .IAccountsPersistence import IAccountsPersistence
from .AccountsMemoryPersistence import AccountsMemoryPersistence
from .AccountsMongoDbPersistence import AccountsMongoDbPersistence



# session_v1.py
from typing import Any, Optional
from datetime import datetime
from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator

class SessionV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        user_id: str = '',
        user_name: Optional[str] = '',
        address: Optional[str] = '',
        client: Optional[str] = ''
    ):
        self.id: str = id or IdGenerator.next_long()
        self.user_id: str = user_id
        self.user_name: Optional[str] = user_name
        self.active: bool = True
        self.open_time: datetime = datetime.utcnow()
        self.request_time: datetime = datetime.utcnow()
        self.close_time: Optional[datetime] = None
        self.address: Optional[str] = address
        self.client: Optional[str] = client
        self.user: Optional[Any] = None
        self.data: Optional[Any] = None

    def to_dict(self):
        return {
            "id": self.id,
            "user_id": self.user_id,
            "user_name": self.user_name,
            "active": self.active,
            "open_time": self.open_time.isoformat() if isinstance(self.open_time, datetime) else self.open_time,
            "request_time": self.request_time.isoformat() if isinstance(self.request_time, datetime) else self.request_time,
            "close_time": self.close_time.isoformat() if isinstance(self.close_time, datetime) else self.close_time,
            "address": self.address,
            "client": self.client,
            "user": self.user.to_dict() if hasattr(self.user, 'to_dict') else self.user,
            "data": self.data 
        }



# -*- coding: utf-8 -*-

__all__ = [
        'AccountsOperations', 
        'ActivitiesOperations', 
        'SettingsOperations', 
        'PasswordsOperations', 
        'RolesOperations', 
        'SessionsOperations', 
        'SessionUserV1', 
        'AuthorizerV1',
        'GroupsOperation',
        'GroupsMembershipsOperation'
        ]

from .AccountsOperations import AccountsOperations
from .ActivitiesOperations import ActivitiesOperations
from .SettingsOperations import SettingsOperations
from .PasswordsOperations import PasswordsOperations
from .RolesOperations import RolesOperations
from .SessionsOperations import SessionsOperations
from .SessionUserV1 import SessionUserV1
from .Authorize import AuthorizerV1
from .GroupsOperation import GroupsOperation
from .GroupsMembershipsOperation import GroupsMembershipsOperation




# -*- coding: utf-8 -*-


class AccountActivityTypeV1:
    AccountCreated: str = 'account created'
    AccountChanged: str = 'account changed'
    AccountDeleted: str = 'account deleted'
    AccountDropped: str = 'account dropped'



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional

from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_components.context import IContext
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data import SessionV1
from .ISessionsPersistence import ISessionsPersistence


class SessionsMongoDbPersistence(IdentifiableMongoDbPersistence, ISessionsPersistence):
    def __init__(self):
        super().__init__('sessions')

    def _convert_to_public(self, value: any) -> any:
        if value is None:
            return None
        
        session = SessionV1(
            id=value.get('_id'),
            user_id=value.get('user_id', ''),
            user_name=value.get('user_name', ''),
            address=value.get('address', ''),
            client=value.get('client', '')
        )
        session.active = value.get('active', True)
        session.open_time = value.get('open_time', datetime.utcnow())
        session.request_time = value.get('request_time', datetime.utcnow())
        session.close_time = value.get('close_time')
        session.user = value.get('user')
        session.data = value.get('data')
        return session

    def _compose_filter(self, filter_params: Optional[FilterParams]) -> dict:
        filter_params = filter_params or FilterParams()
        criteria = []

        id = filter_params.get_as_nullable_string('id')
        if id:
            criteria.append({'_id': id})

        user_id = filter_params.get_as_nullable_string('user_id')
        if user_id:
            criteria.append({'user_id': user_id})

        active = filter_params.get_as_nullable_boolean('active')
        if active is not None:
            criteria.append({'active': active})

        from_time = filter_params.get_as_nullable_datetime('from_time')
        if from_time:
            criteria.append({'request_time': {'$gte': from_time}})

        to_time = filter_params.get_as_nullable_datetime('to_time')
        if to_time:
            criteria.append({'request_time': {'$lt': to_time}})

        return {'$and': criteria} if criteria else {}

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self._compose_filter(filter_params), paging, sort={'request_time': -1})

    def create(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        now = datetime.utcnow()
        item.open_time = now
        item.request_time = now
        return super().create(context, item)

    def update(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        item.request_time = datetime.utcnow()
        return super().update(context, item)

    def close_expired(self, context: Optional[IContext], request_time: datetime):
        criteria = {'request_time': {'$lt': request_time}, 'active': True}
        update = {
            '$set': {
                'active': False,
                'request_time': datetime.utcnow(),
                'close_time': datetime.utcnow(),
                'user': None,
                'data': None
            }
        }
        result = self._collection.update_many(criteria, update)
        if result.modified_count > 0:
            self._logger.debug(context, f'Closed {result.modified_count} expired sessions')



# -*- coding: utf-8 -*-
from datetime import datetime
import json
import bottle
import asyncio

from typing import List
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_commons.errors import BadRequestException
from pip_services4_data.query import FilterParams
from pip_services4_components.context import Context

from eic_aichat_users.groupmemberships.data.GroupMembershipV1 import GroupMembershipV1
from eic_aichat_users.groupmemberships.logic.IGroupMembershipsService import IGroupMembershipsService
from eic_aichat_users.groups.logic.IGroupsService import IGroupsService
from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1


# TODO
class GroupsMembershipsOperation(RestOperations):
    def __init__(self):
        super().__init__()
        self._group_memberships: IGroupMembershipsService = None
        self._group: IGroupsService = None
        self._dependency_resolver.put('groupmemberships', Descriptor('aichatusers-groupmemberships', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put('groups', Descriptor('aichatusers-groups', 'service', '*', '*', '1.0'))

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._group_memberships = self._dependency_resolver.get_one_required('groupmemberships')
        self._group = self._dependency_resolver.get_one_required('groups')

    def get_memberships(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        filter_params = self._get_filter_params()
        paging = self._get_paging_params()

        try:
            page = self._group_memberships.get_memberships(context, filter_params, paging)
            return self._send_result(page)
        except Exception as err:
            return self._send_error(err)

    def get_membership_by_id(self, membership_id: str):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, membership_id)
            # if membership is None or membership.id == "":
            #     raise BadRequestException(self._get_trace_id(), "WRONG_MEMBERSHIP_ID", "Membership id does not exist")
            # if membership.profile_id != user_id:
            #     raise BadRequestException(self._get_trace_id(), "WRONG_USER", "Profile id does not match user id")
            
            return self._send_result(membership)
        except Exception as err:
            return self._send_error(err)

    def create_membership(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        groupMembership_data = data if isinstance(data, dict) else json.loads(data or '{}')
        groupMembership = GroupMembershipV1(**groupMembership_data)

        try:
            group = self._group.get_group_by_id(context, groupMembership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            memberships = self._group_memberships.create_membership(context, groupMembership)
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def update_membership(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
    
        data = bottle.request.json
        groupMembership_data = data if isinstance(data, dict) else json.loads(data or '{}')
        groupMembership = GroupMembershipV1(**groupMembership_data)

        try:
            group = self._group.get_group_by_id(context, groupMembership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            memberships = self._group_memberships.update_membership(context, groupMembership)
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def delete_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_MEMBERSHIP_ID", "Membership id does not exist")
            if membership.profile_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "Profile id does not match user id")
            
            memberships = self._group_memberships.delete_membership_by_id(context, id)
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def delete_membership_by_filter(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        try:
            res = self._group.delete_groups_by_filter(context, filter_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def activate_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_ID", "Id does not exist")
            
            group = self._group.get_group_by_id(context, membership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            membership.active = True
            membership.member_since = datetime.utcnow()
            
            res = self._group_memberships.update_membership(context, membership)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def deactivate_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_ID", "Id does not exist")
            
            group = self._group.get_group_by_id(context, membership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            membership.active = False
            
            res = self._group_memberships.update_membership(context, membership)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def register_routes(self, controller: RestController, auth: AuthorizerV1):
            controller.register_route_with_auth('get', '/users/groups/memberships', None, auth.signed(), lambda: self.get_memberships())
            
            controller.register_route_with_auth('get', '/users/groups/:id/memberships', None, auth.signed(), lambda id: self.get_membership_by_id(id))

            controller.register_route_with_auth('post', '/users/groups/memberships', None, auth.signed(), lambda: self.create_membership())

            controller.register_route_with_auth('put', '/users/groups/memberships', None, auth.signed(), lambda: self.update_membership())

            controller.register_route_with_auth('delete', '/users/groups/:id/memberships', None, auth.signed(), lambda id: self.delete_membership_by_id(id))

            controller.register_route_with_auth('delete', '/users/groups/memberships', None, auth.admin(), lambda: self.delete_membership_by_filter())

            controller.register_route_with_auth('post', '/users/groups/:id/memberships/activate', None, auth.signed(), lambda id: self.activate_membership_by_id(id))

            controller.register_route_with_auth('post', '/users/groups/:id/memberships/deactivate', None, auth.signed(), lambda id: self.deactivate_membership_by_id(id))
            
        



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import List, Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data.UserRolesV1 import UserRolesV1


class IRolesService(ABC):

    @abstractmethod
    def get_roles_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                            paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_roles_by_id(self, context: Optional[IContext], user_id: str) -> Optional[List[str]]:
        raise NotImplementedError()

    @abstractmethod
    def set_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def grant_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def revoke_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def authorize(self, context: Optional[IContext], user_id: str, roles: List[str]) -> bool:
        raise NotImplementedError()



# -*- coding: utf-8 -*-

__all__ = ['IGroupMembershipsService', 'GroupMembershipsService']

from .IGroupMembershipsService import IGroupMembershipsService
from .GroupMembershipsService import GroupMembershipsService



# -*- coding: utf-8 -*-
import json

import bottle
from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema, ArraySchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context

from eic_aichat_users.activities.data import PartyActivityV1, PartyActivityV1Schema
from eic_aichat_users.activities.logic.IActivitiesService import IActivitiesService
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService
from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1


class ActivitiesOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._activities_service: IActivitiesService = None
        self._dependency_resolver.put("activities-service", Descriptor('aichatusers-activities', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._activities_service = self._dependency_resolver.get_one_required('activities-service')

    def get_activities(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_activities() invoked")

        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._activities_service.get_activities(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
    
    def get_party_activities(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_party_activities() invoked")

        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        party_id = self.get_param("party_id")
        filter_params.set_as_object("party_id", party_id)
        try:
            res = self._activities_service.get_activities(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def log_activity(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------log_activity() invoked")

        activity = self._get_data()
        activity = None if not activity else PartyActivityV1(**activity)
        try:
            res = self._activities_service.log_activity(context, activity)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def log_activities(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------log_activities() invoked")

        activities = self._get_data()
        activities = None if not activities else [PartyActivityV1(**activity) for activity in activities]
        try:
            res = self._activities_service.batch_log_activities(context, activities)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
    

    def register_routes(self, controller: RestController, auth: AuthorizerV1):
        pass
        # controller.register_route('get', 'users/activities', None, self.get_activities)

        # controller.register_route('get', 'users/<party_id>/activities', None, self.get_party_activities)

        # controller.register_route('post', 'users/activities', PartyActivityV1Schema(),
        #                           self.log_activity)
        
        # controller.register_route('post', 'users/activities/batch', ArraySchema(PartyActivityV1Schema()),
        #                           self.log_activities)

        
